#!/bin/bash
##### {COPYRIGHT-TOP} ##############################################
# IBM Confidential
# OCO Source Materials
# 5724-V51
#
# (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
##### {COPYRIGHT-END} ##############################################
## %DISCLAIMER%

# The purpose of this script is to migrate the database schema from 5.3.2 to 6.0.0
# Please consult InfoSphere MDM Server for PIM Support before attempting to use this script
. ${TOP}/bin/get_params.sh


if [ -z "$TOP" -o ! -r "$TOP" ]; then
   ${ECHO} TOP variable not defined
   exit 1
fi

# Set the 5.3.2-FP009 database to be migrated to Glendale

${ECHO}
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO} "**"
${ECHO} Create tables and triggers for DCL component.
${ECHO} Backup the DB before you run this shell scripts.
${ECHO} "**"
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO}

. ${TOP}/src/db/schema/util/set_db_localized.sh "$@"
_ERROR_LOGS=$TOP/dcl/logs/dclerrfile.log

EXISTS=1;
NOTEXISTS=0;
count=0;

if [ -d $TOP/tmp ]; then
  TMPDIREXISTS=$EXISTS
else
  TMPDIREXISTS=$NOTEXISTS
fi


if [ -d $TOP/dcl/logs ]; then
  rm -f ${_ERROR_LOGS}
else
  mkdir $TOP/dcl/logs
fi

case ${CCD_DB} in
	db2)
		DB_TYPE="DB2"
		;;
	oracle)
		DB_TYPE="Oracle"
		;;
esac


getErrorCountInLogFile()
{
    if [ ! -f $_ERROR_LOGS ]; then
        errorCount=0
    else
        errorCount=`cat $_ERROR_LOGS | wc -c`
    fi
}

DoStartAndPrepare()
{
    ${ECHO} 
    ${ECHO} $1
    ${ECHO} 

    getErrorCountInLogFile
    initial=$errorCount
}

DoEndAndReportError()
{

    getErrorCountInLogFile
    final=$errorCount

    if [ ! $initial -eq $final ]; then

        ${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        ${ECHO} $1
        ${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        ${ECHO} "COMMON_INFO_CHECK_ERROR_LOG_FILE_SH" $_ERROR_LOGS
        
        exit -1

    fi
        ${ECHO} "---------------------------------------------"
        ${ECHO} $2
        ${ECHO} "---------------------------------------------"

}



# Define the functions for schema changes

createDCLTables()
{	
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New Tables"   
	
	cd $TOP/dcl/DDL/${DB_TYPE}/tables

	for DDL in `ls TDCL_*.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "COMMON_INFO_FAILED_SH New Tables" "COMMON_INFO_SUCCEEDED_SH New Tables"	
	done
}


createDCLConstraints()
{	
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New Tables"   
	
	cd $TOP/dcl/DDL/${DB_TYPE}/constraints

	for DDL in `ls *.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "COMMON_INFO_FAILED_SH New constraints" "COMMON_INFO_SUCCEEDED_SH New Tables"	
	done
}

createDB2DCLTriggers()
{	
	echo ${CONNECTION}
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New Tables"   
	
	cd $TOP/dcl/DDL/${DB_TYPE}/triggers
	

	echo ${CCD_SQLPLUS} ${CONNECTION}
	db2 connect to $DB_SERVICE_NAME user $DB_USER_NAME using $DB_USER_PWD 
	for DDL in `ls *.sql 2>/dev/null`
	do
	echo "executing $DDL" >> ${_ERROR_LOGS}
	#. ${DB_DIR}/util/run_db_cmd.sh --debug=yes --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
	sed "s/<SCHEMA>/${DB_USER_NAME}/g" $DDL| db2 -td@ >> ${_ERROR_LOGS}
	#db2 -tfd@ $DDL
	#DoEndAndReportError  "COMMON_INFO_FAILED_SH New constraints" "COMMON_INFO_SUCCEEDED_SH New Tables"	
	done
	db2 commit
}

createOracleDCLTriggers()
{
	cd $TOP/dcl/DDL/Oracle/triggers
	for DDL in `ls *.sql 2>/dev/null`
	do
	echo $DDL >> ${_ERROR_LOGS}
	. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
	done
}

createDCLSequences()
{
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New sequences"   
	
	cd $TOP/dcl/DDL/${DB_TYPE}/sequences

	for DDL in `ls *.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "COMMON_INFO_FAILED_SH New sequences" "COMMON_INFO_SUCCEEDED_SH New sequences"	
	done
}

populateDefaultConfigData()
{
	DoStartAndPrepare "Populating PLP Default data"   
	
	cd $TOP/dcl/DML/${DB_TYPE}

	for DML in `ls *.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DML --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "Populating PLP Default data"   
	done
}


# Call the functions to make changes in the schema
createDCLTables
createDCLSequences
createDCLConstraints

#     if [ ! $initial -eq $final ]; then
echo "${CCD_DB}"
case ${CCD_DB} in
	db2)
		echo "${CCD_DB}"
		createDB2DCLTriggers
		;;
	oracle)
		echo "${CCD_DB}"
		createOracleDCLTriggers
		;;
esac

populateDefaultConfigData
#


