#!/bin/bash
##### {COPYRIGHT-TOP} ##############################################
# IBM Confidential
# OCO Source Materials
# 5724-V51
#
# (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
##### {COPYRIGHT-END} ##############################################
## %DISCLAIMER%

# The purpose of this script is to create tables for exporting entity changes to MDMS
# Please consult InfoSphere MDM Server for PIM Support before attempting to use this script
. ${TOP}/bin/get_params.sh


if [ -z "$TOP" -o ! -r "$TOP" ]; then
   ${ECHO} TOP variable not defined
   exit 1
fi

${ECHO}
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO} "**"
${ECHO} Create tables for MDMExport component.
${ECHO} Backup the DB before you run this shell scripts.
${ECHO} "**"
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO}

. ${TOP}/src/db/schema/util/set_db_localized.sh "$@"
_ERROR_LOGS=$TOP/export_dcl/logs/dclerrfile.log

EXISTS=1;
NOTEXISTS=0;
count=0;

if [ -d $TOP/tmp ]; then
  TMPDIREXISTS=$EXISTS
else
  TMPDIREXISTS=$NOTEXISTS
fi


if [ -d $TOP/export_dcl/logs ]; then
  rm -f ${_ERROR_LOGS}
else
  mkdir $TOP/export_dcl/logs
fi

case ${CCD_DB} in
	db2)
		DB_TYPE="DB2"
		;;
	oracle)
		DB_TYPE="Oracle"
		;;
esac


getErrorCountInLogFile()
{
    if [ ! -f $_ERROR_LOGS ]; then
        errorCount=0
    else
        errorCount=`cat $_ERROR_LOGS | wc -c`
    fi
}

DoStartAndPrepare()
{
    ${ECHO} 
    ${ECHO} $1
    ${ECHO} 

    getErrorCountInLogFile
    initial=$errorCount
}

DoEndAndReportError()
{

    getErrorCountInLogFile
    final=$errorCount

    if [ ! $initial -eq $final ]; then

        ${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        ${ECHO} $1
        ${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        ${ECHO} "COMMON_INFO_CHECK_ERROR_LOG_FILE_SH" $_ERROR_LOGS
        
        exit -1

    fi
        ${ECHO} "---------------------------------------------"
        ${ECHO} $2
        ${ECHO} "---------------------------------------------"

}



# Define the functions for schema changes

createMDMExportTables()
{	
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New Tables"   
	
	cd $TOP/export_dcl/DDL/${DB_TYPE}/tables

	for DDL in `ls TEXP_*.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "COMMON_INFO_FAILED_SH New Tables" "COMMON_INFO_SUCCEEDED_SH New Tables"	
	done
}


createMDMExportConstraints()
{	
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New Constraints"   
	
	cd $TOP/export_dcl/DDL/${DB_TYPE}/constraints

	for DDL in `ls *.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "COMMON_INFO_FAILED_SH New constraints" "COMMON_INFO_SUCCEEDED_SH New Constraints"	
	done
}

createMDMExportSequences()
{
	DoStartAndPrepare "COMMON_INFO_UPDATING_SH New sequences"   
	
	cd $TOP/export_dcl/DDL/${DB_TYPE}/sequences

	for DDL in `ls *.sql 2>/dev/null`
	do
		. ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes 2>> $_ERROR_LOGS
		
	DoEndAndReportError  "COMMON_INFO_FAILED_SH New sequences" "COMMON_INFO_SUCCEEDED_SH New sequences"	
	done
}


# Call the functions to make changes in the schema
createMDMExportTables
createMDMExportSequences
createMDMExportConstraints
