#!/bin/bash
#set -x

 Usage of the file
if [[ $# -ne 3 ]]
then
echo "Usage "
echo $0 " --db=dbname --instowner=instanceowner --cpath=containerpath"
echo $0 " --db=pimdb --instowner=db2inst1 --cpath=/u01"
 exit 0
fi
echo "Last update of script: MDMPIM 6.0.0-FP010 2010-02-23"
date > ~/db_creation.log

DBName=`echo $1 |cut -d "=" -f2`;export DBName
INSTOwner=`echo $2 |cut -d "=" -f2`;export INSTOwner
DBDir=`echo $3 |cut -d "=" -f2`;export DBDir

mkdir -p ${DBDir}/db2data/${DBName}/logs

# This script is used to create MDMPIM database

echo "------------------------------------------------------------------------"
echo "Attaching to the instance ${INSTowner} and starting the db instance"
echo "*** IGNORE SQL1026N message if the db manager is already active ***"
echo "------------------------------------------------------------------------"

db2 -v "attach to ${INSTOwner}"
db2start

echo "------------------------------------------------------------------------"
echo "Creating the database ${DBName} with UTF-8 Character Set"
echo "------------------------------------------------------------------------"

db2 -v "CREATE DATABASE ${DBName} AUTOMATIC STORAGE YES ON '${DBDir}/db2data/${DBName}' USING CODESET UTF-8 TERRITORY US"
db2 -v "activate database ${DBName}"

echo "------------------------------------------------------------------------"
echo "Creating MDMPIM specific bufferpools"
echo "*** IGNORE SQL20189W messages ***" 
echo "------------------------------------------------------------------------"

db2 -v "connect to ${DBName}"
cat << EOF > ~/bp-dbm-db-cfg-3tsp.sql
----- Buffer pool configuration
CREATE BUFFERPOOL USERSBP SIZE AUTOMATIC PAGESIZE 16K;
CREATE BUFFERPOOL INDXBP SIZE AUTOMATIC PAGESIZE 16K;
CREATE BUFFERPOOL BLOBBP SIZE AUTOMATIC PAGESIZE 16K;
CREATE BUFFERPOOL TEMPUSRBP SIZE AUTOMATIC PAGESIZE 16K;
CREATE BUFFERPOOL TEMPSYSBP SIZE AUTOMATIC PAGESIZE 16K;
CREATE BUFFERPOOL TEMPUSRBP32 SIZE AUTOMATIC PAGESIZE 32K;
CREATE BUFFERPOOL TEMPSYSBP32 SIZE AUTOMATIC PAGESIZE 32K;

----- Database Manager Configuration
update dbm cfg using MON_HEAP_SZ 5120;
update dbm cfg using QUERY_HEAP_SZ 1024;
update dbm cfg using ASLHEAPSZ 1024;
update dbm cfg using MAXAGENTS 120;

----- Database configuration
update db cfg using SELF_TUNING_MEM ON;
update db cfg using DFT_QUERYOPT 5;
update db cfg using DBHEAP 20000;
update db cfg using CATALOGCACHE_SZ 5120;
update db cfg using LOGBUFSZ 4096;
update db cfg using UTIL_HEAP_SZ 5120;
update db cfg using APP_CTL_HEAP_SZ 512;
update db cfg using STMTHEAP 2048;
update db cfg using APPLHEAPSZ 1024;
update db cfg using STAT_HEAP_SZ 10240;
update db cfg using LOCKTIMEOUT 60;
update db cfg using NUM_IOCLEANERS 7;
update db cfg using NUM_IOSERVERS 8;
update db cfg using MAXAPPLS 120;
update db cfg using AVG_APPLS 5;
update db cfg using MAXFILOP 640;
----- Log file configuration 
update db cfg using LOGFILSIZ 3000;
update db cfg using LOGPRIMARY 40;
update db cfg using LOGSECOND 2;
EOF

db2 -tvf ~/bp-dbm-db-cfg-3tsp.sql >> ~/db_creation.log

echo "------------------------------------------------------------------------"
echo "Creating MDMPIM Specific tablespaces"
echo "*** It takes time to create the tablespaces. ***"
echo "*** Be patient. DO NOT kill the statement while running ***"
echo "------------------------------------------------------------------------"


cat << EOF > ~/tablespaces.sql
CREATE REGULAR TABLESPACE USERS PAGESIZE 16K MANAGED BY AUTOMATIC STORAGE BUFFERPOOL USERSBP NO FILE SYSTEM CACHING;
CREATE REGULAR TABLESPACE INDX PAGESIZE 16K MANAGED BY AUTOMATIC STORAGE BUFFERPOOL INDXBP NO FILE SYSTEM CACHING;
CREATE REGULAR TABLESPACE BLOB_TBL_DATA PAGESIZE 16K MANAGED BY AUTOMATIC STORAGE BUFFERPOOL BLOBBP NO FILE SYSTEM CACHING;
CREATE USER TEMPORARY TABLESPACE TEMP_USER PAGESIZE 16K MANAGED BY SYSTEM USING ('/${DBDir}/${DBName}/usertemp01') BUFFERPOOL TEMPUSRBP;
CREATE SYSTEM TEMPORARY TABLESPACE TEMP_SYSTEM PAGESIZE 16K MANAGED BY SYSTEM USING ('/${DBDir}/${DBName}/systemtemp01') BUFFERPOOL TEMPSYSBP;
CREATE USER TEMPORARY TABLESPACE TEMP_USER32 PAGESIZE 32K MANAGED BY SYSTEM USING ('/${DBDir}/${DBName}/usertemp32') BUFFERPOOL TEMPUSRBP32;
CREATE SYSTEM TEMPORARY TABLESPACE TEMP_SYSTEM32 PAGESIZE 32K MANAGED BY SYSTEM USING ('/${DBDir}/${DBName}/systemtemp32') BUFFERPOOL TEMPSYSBP32;
EOF
db2 -tvf ~/tablespaces.sql >> ~/db_creation.log

echo "------------------------------------------------------------------------"
echo "Updating Transaction Logs Specific Configuration parameters"
echo "*** IGNORE SQL1363W messages ***"
echo "------------------------------------------------------------------------"

db2 -v "update db cfg for ${DBName} using NEWLOGPATH ${DBDir}/db2data/${DBName}/logs" >> ~/db_creation.log
db2 -v "connect reset"

echo "------------------------------------------------------------------------"
echo "Updating DB2 Registry Variables"
echo "------------------------------------------------------------------------"

db2set DB2ENVLIST=EXTSHM
db2set DB2COMM=tcpip
db2set DB2CODEPAGE=1208
db2set DB2AUTOSTART=yes
db2set DB2_SKIPINSERTED=ON
db2set DB2_EVALUNCOMMITTED=ON
db2set DB2_SKIPDELETED=ON
db2set DB2_PARALLEL_IO=*

echo "------------------------------------------------------------------------"
echo "Updating DB2 Registry Variables is DONE"
echo "Bouncing the database to apply all the parameter changes"
echo "------------------------------------------------------------------------"

db2stop force
db2start

echo "**************************************************************************"
echo "*** Following statement is going to take time to activate the database ***"
echo "*** DO NOT INTERRRUPT the command and DO NOT KILL the statement ***"
echo "**************************************************************************"

db2 -v "activate database ${DBName}"

echo "------------------------------------------------------------------------"
echo "${DBName} Database creation script completed. "
echo "Check logfile ~/db_creation.log for errors"
echo "------------------------------------------------------------------------"
