/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/



/**
 *  Copyright IBM Corp. and other(s) 2000, 2008 All Rights Reserved. 
 *
 * The following sample of source code ("Sample Code") is owned by International Business Machines Corporation or 
 * one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.
 * You may use, copy, modify, and distribute the Sample Code in any form without payment to IBM, for the purpose of assisting you in the development of your applications.

 * The Sample Code is provided to you on an "AS IS" basis, without warranty of any kind. 
 * IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * Some jurisdictions do not allow for the exclusion or limitation of implied warranties, so the above limitations or exclusions may not apply to you. 
 * IBM shall not be liable for any damages you suffer as a result of using, copying, modifying or distributing the Sample Code, even if IBM has been advised of the possibility of such damages.
 **/
package com.ibm.ccd.loader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.SQLException;

import com.ibm.pim.context.Context;
import com.ibm.pim.context.PIMContextFactory;
import com.ibm.pim.utils.Logger;

public class DataLoader{

	private static final String OPTION_CSV = "csv";
	private static final String OPTION_TABLE = "table";	
	private static final String DEFAULT_LOGGER_NAME = "DEFAULT";

	Context context;  
	Logger logger;

	/***
	 * 
	 * @param propertyFile -path of the property file
	 * @param dataFile - path of the data file
	 * @param option - option either 'csv' or table'
	 * @param jobId - process data corresponding to the jobId and between the startRecordId and stoprecordIdwhen the option is 'table'
	 * @param startRecordId - is used while loading data from the table
	 * @param stopRecordId - is used while loading data from the table.
	 * @throws FileNotFoundException when file (property file or data file) does not Exist
	 * @throws IOException when Exception occurs while reading database properties from common.properties
	 * @throws PIMAuthorizationException when Invalid Authorization information is given
	 * @throws SQLException when there is an Exception while loading data from the database tables.
	 * @throws Exception
	 */
	public void createLoader(String propertyFile,String dataFile,String option,Integer jobId,Integer startRecordId, Integer stopRecordId) throws FileNotFoundException,IOException,SQLException,Exception
	{			
		context =PIMContextFactory.getCurrentContext();
		logger = context.getLogger(DEFAULT_LOGGER_NAME);
		//if the input option is csv, invoke csvLoader
		if(option != null && !option.trim().equals("") && option.equals(OPTION_CSV))
		{
			if(propertyFile == null || dataFile ==null)
			{
				logger.logError("Property file and data file path cannot be null while loading data from CSV");
				return;
			}
			CSVLoader csvLoader = new CSVLoader(propertyFile,dataFile);
			csvLoader.loadData();
		}

		//if the input option is table , invoke DatabaseLoader
		else if(option != null && !option.trim().equals("") && option.equals(OPTION_TABLE))
		{
			if(propertyFile == null || jobId ==null || startRecordId == null || stopRecordId ==null)
			{
				logger.logError("Property file path , job id, start record id and Stop record id cannot be null while loading data from tables");   
				return;
			}
			DatabaseLoader tableLoader = new DatabaseLoader(propertyFile,jobId,startRecordId,stopRecordId);
			tableLoader.loadData();
		}

		else
		{
			logger.logError("Invalid Option specified to load data"); 
			return;
		}		
	}


}
