/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/



/**
 * � Copyright IBM Corp. and other(s) 2000, 2008 All Rights Reserved. 
 *
 * The following sample of source code ("Sample Code") is owned by International Business Machines Corporation or 
 * one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.
 * You may use, copy, modify, and distribute the Sample Code in any form without payment to IBM, for the purpose of assisting you in the development of your applications.

 * The Sample Code is provided to you on an "AS IS" basis, without warranty of any kind. 
 * IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * Some jurisdictions do not allow for the exclusion or limitation of implied warranties, so the above limitations or exclusions may not apply to you. 
 * IBM shall not be liable for any damages you suffer as a result of using, copying, modifying or distributing the Sample Code, even if IBM has been advised of the possibility of such damages.
 **/
package com.ibm.ccd.loader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import com.ibm.pim.attribute.AttributeDefinition;
import com.ibm.pim.attribute.AttributeInstance;
import com.ibm.pim.attribute.AttributePathHelper;
import com.ibm.pim.catalog.Catalog;
import com.ibm.pim.catalog.CatalogManager;
import com.ibm.pim.catalog.item.Item;
import com.ibm.pim.collaboration.CollaborationArea;
import com.ibm.pim.collaboration.CollaborationAreaManager;
import com.ibm.pim.collaboration.ItemCollaborationArea;
import com.ibm.pim.common.ExtendedValidationErrors;
import com.ibm.pim.common.ProcessingOptions;
import com.ibm.pim.common.ValidationError;
import com.ibm.pim.common.exceptions.PIMAuthorizationException;
import com.ibm.pim.common.exceptions.PIMInternalException;
import com.ibm.pim.common.exceptions.PIMInvalidPathException;
import com.ibm.pim.context.Context;
import com.ibm.pim.context.PIMContextFactory;
import com.ibm.pim.hierarchy.Hierarchy;
import com.ibm.pim.hierarchy.HierarchyManager;
import com.ibm.pim.hierarchy.category.Category;
import com.ibm.pim.spec.PrimarySpec;
import com.ibm.pim.spec.Spec;
import com.ibm.pim.spec.SpecManager;
import com.ibm.pim.utils.Logger;

public abstract class DataProcessor {

	private static final String DEFAULT_ACTION_MODE = "Update";
	private static final String CATALOG = "Catalog";
	private static final String CATEGORY = "Category";
	private static final String ACTION_MODE = "ActionMode";
	private static final String CREATE_ACTION_MODE = "Create";
	private static final String UPDATE_ACTION_MODE = "Update";
	private static final String DELETE_ACTION_MODE = "Delete";

	private static final String CREATE_FAILED_ITEM_EXISTS = "Create failed : Item already exists";
	private static final String DELETE_FAILED_NO_ITEM= "Delete failed : Item does not exist";
	private static final String UPDATE_FAILED_ITEM_CHECKED_OUT= "Update failed : Item has checked out attributes";
	private static final String DELETE_FAILED_ITEM_CHECKED_OUT="Delete failed : Item has checked out attributes ";

	private static final String STATUS_INVALID_CATALOG = "Error : Invalid Catalog";
	private static final String STATUS_ITEM_DOES_NOT_EXIST = "Error : Item does not exist";
	private static final String STATUS_ITEM_HAS_CHECKED_OUT_ATTRIBUTES = "Error : Item has checked out attributes";
	private static final String STATUS_VALIDATION_ERROR = "Error : Validation Error";
	private static final String STATUS_ITEM_ALREADY_EXISTS = "Error : Item already exists";
	private static final String STATUS_INVALID_PATH = "Error : Invalid Path";
	private static final String STATUS_SETTING_RELATION_ATTRIBUTE = "Error: Setting Relationship Attribute";
	private static final String STATUS_ITEM_SAVED = "Processed : Item Save";
	private static final String STATUS_INVALID_INPUT_FORMAT = "Error:Invalid Input Format";
	//when the item does not have a primary key in the test data and cannot obtain a primary key value(the primary key is not a sequence attribute , has no value rule associated)
	private static final String STATUS_NO_PRIMARY_KEY = "Error: Item does not have Primay Key";
	// For the items which does not have primary key in the test data , but obtain the primary key later by value rule or by sequence attribute, you need to look for the status in the log
	private static final String STATUS_CHECK_STATUS_IN_LOGS = "check the status of this item in the logs";

	private static final String EXCEPTION_LOGGER_NAME = "JavaAPI_Exception";

	Context context;  
	//Properties defined in the property file
	//defaultCatalog - Item imported into this catalog, when the item is not associated with a catalog in the data file
	private String defaultCatalog;
	//checkout all the imported items in to this collaboration area
	private String collaborationAreaName;
	//disable procesing options boolean - used to specify whether to turn down the processing options or not
	private boolean disableProcessingOptions;    
	//checkoutItemSize- size of the collection of items to be checked out into the collaboration area
	private int checkoutItemSize;
	//itemSaveBatchSize - size of the items to be saved as a batch
	private int itemSaveBatchSize;

	//hashmap to hold the Catalog Object and the corresponding Primary Key of the items in the catalog   
	private HashMap<String,String> hmCatalogItemPrimaryKey = new HashMap<String,String>();
	private PrimarySpec primarySpec = null;     

	//A collection to hold the items to be checked out
	private ArrayList<Item> checkoutItems= new ArrayList<Item>();
	private boolean collabExtn = false;	 
	private ItemCollaborationArea itmColArea;

//	HashMap to hold the Catalog, CollaborationArea , Spec objects
	private HashMap<String,Catalog> hmCatalogObj = new HashMap<String,Catalog>();
	private HashMap<String,CollaborationArea>  hmCollaborationAreaObj= new HashMap<String,CollaborationArea>();
	private HashMap<String,Spec>  hmSpecObj= new HashMap<String,Spec>();

	//HashMap to hold the categoryObjects    
	private HashMap<String,Category> hmPathToCategories = new HashMap<String,Category>();  

	//HashMap to hold the spec path and its type
	private HashMap<String,AttributeDefinition.Type> specPathToType =new HashMap<String,AttributeDefinition.Type>();

	//To update the status table
	private PreparedStatement insertStatusQuery;
	private String status;

	//Variables to process each row
	private CatalogManager ctgMgr = null;
	AttributePathHelper attribPathHelper = null;
	SpecManager spcManager = null;
	HierarchyManager hierMgr = null;
	CollaborationAreaManager caMgr = null;
	Logger logger = null;
	Logger exceptionLogger = null;


	int previousRecordId = -1;	
	ArrayList<String> categoryNames = new ArrayList<String>();
	String catalogName = null;
	//actionMode can be different for each item, by default it is update
	String actionMode = DEFAULT_ACTION_MODE;
	LinkedHashMap<String,String> pathValue = new LinkedHashMap<String,String>();

	//This HashMap has the catalog as the key and the hashmap of the items added to the Batch and the object RecordInfo for the item as its value.
	private HashMap<Catalog, HashMap<String, List<RecordInfo>>> hmCatalogToBatchedItems = new HashMap<Catalog, HashMap<String, List<RecordInfo>>>(); 
	//list of items which has no pk mentioned in the testdata and can obtain pk from value rule or when pk is of item sequence, this Array list is used for status table updation
	private ArrayList<RecordInfo> listRecordInfoForItemsWithNoPk = new ArrayList<RecordInfo>();

	//ArrayList of Batched Catalogs
	private ArrayList<Catalog> batchedCatalogs = new ArrayList<Catalog>();

	private HashMap<Catalog,Integer> hmCatalogToBatchedItemCount = new HashMap<Catalog,Integer>(); 


	/***
	 *  The class <code>RecordInfo</code> is used to store the JobId and the RecordId info from the data file
	 */
	public class RecordInfo
	{
		public int jobId;
		public int recordId;
	}

	public DataProcessor()
	{
	}  

	public DataProcessor(String propFile)throws Exception
	{  
		initContextAndManagers();
		initProperties(propFile);
        String insertStatusQueryStr = "insert into tldr_stt_status_table values(?,?,?,?)";
		insertStatusQuery = ProcessorUtils.getConnection().prepareStatement(insertStatusQueryStr);
	}

	private void initContextAndManagers()throws Exception
	{
		try
		{
			context = PIMContextFactory.getCurrentContext();
			ctgMgr = context.getCatalogManager();
			attribPathHelper = context.getAttributePathHelper();
			spcManager = context.getSpecManager();
			hierMgr = context.getHierarchyManager();
			caMgr = context.getCollaborationAreaManager();
			logger = context.getDefaultLogger();
			exceptionLogger = context.getLogger(EXCEPTION_LOGGER_NAME);

		}
		catch (Exception e)
		{
			logger.logError("Invalid Authorization Information"+e);
			throw e;
		}	
	}


	private void initProperties(String propFile)throws FileNotFoundException,IOException,Exception
	{
		PropertyLoader propLoader = new PropertyLoader();
		PropertyLoader.PropertyFileProperties properties = propLoader.readPropertyFile(propFile);
		defaultCatalog = properties.defaultCatalog;
		collaborationAreaName = properties.collaborationAreaName;
		disableProcessingOptions = properties.disableProcessingOptions;
		checkoutItemSize = properties.checkoutItemSize;
		itemSaveBatchSize = properties.itemSaveBatchSize;
	}


	/***
	 * This method is implemented either by the CSVLoader or the DatabaseLoader class accordingly 
	 * @throws FileNotFoundException - When the file(data file or the property file) does not exist
	 * @throws IOException - occurs when parsing the files
	 * @throws PIMAuthorizationException - when user is not Aythorized to do the operation
	 * @throws SQLException - while loading data from the table
	 */
	public abstract void loadData()throws IOException,FileNotFoundException,SQLException,Exception; 

	/***
	 * Processes each line of the data file
	 * @param jobId
	 * @param currentRecordId
	 * @param name
	 * @param value
	 * @throws PIMAuthorizationException- when user is not Authorized to do the operation
	 * @throws SQLException - while loading data from the table
	 * @throws Exception - Error when getting the context
	 */
	public void processRow(int jobId, int currentRecordId, String name, String value) throws SQLException,Exception
	{
		if (currentRecordId != previousRecordId)
		{
			if (previousRecordId != -1)
			{
				processRecord(jobId, currentRecordId, catalogName, categoryNames, actionMode, pathValue);				
			}
			catalogName = null;
			categoryNames = new ArrayList<String>();
			pathValue = new LinkedHashMap<String,String>();
			actionMode = DEFAULT_ACTION_MODE;
		}
		if (name.equals(CATALOG))
		{
			catalogName = value;
		}
		else if (name.equals(CATEGORY))
		{
			categoryNames.add(value);
		}
		else if (name.equals(ACTION_MODE))
		{
			actionMode = value;
		}
		else
		{
			if (name != null && !name.equals(""))
			{
				pathValue.put(name, value);
			}
		}
		previousRecordId = currentRecordId;
	}

	/***
	 * Used to do functions once the EOF (Data file ) is reached
	 * @param jobId
	 * @param currentRecordId
	 * @throws PIMAuthorizationException -when user is not Authorized to do the operation
	 * @throws SQLException - while loading data from the table
	 * @throws Exception - Error when getting the context
	 */
	public void finishUpLoadData(int jobId, int currentRecordId)throws SQLException,Exception
	{
		List<ExtendedValidationErrors> validationErrors = null ;
		//this is for processing the last record
		if (previousRecordId != -1)
		{
			processRecord(jobId, currentRecordId, catalogName, categoryNames, actionMode, pathValue);
		}

		for(Iterator it = batchedCatalogs.iterator();it.hasNext();)
		{
			Catalog ctg = (Catalog)it.next();
			validationErrors = ctg.stopBatchProcessing();
			if(validationErrors!= null)
			{
				updateStatusTableForItemSavedAsBatch(validationErrors,ctg);
			}	
			it.remove();
		}

		//checkout the items once the EOF is reached
		batchCheckout();
	}

	private Catalog getCatalogFromCache(String sCatalog, boolean setProcessingOptions)
	{
		//Caching the catalog objects and also setting the processing options for that catalog at the same time
		Catalog catalog = hmCatalogObj.get(sCatalog);
		if(catalog==null)
		{	        	 			  
			//get the catalog object			 
			catalog = ctgMgr.getCatalog(sCatalog);
			if(catalog != null)
			{
				hmCatalogObj.put(sCatalog,catalog);             
				//setting the processsing options while populating the catalog object
				if(setProcessingOptions == true)
				{
					if (disableProcessingOptions == true)
					{					
						ProcessingOptions procOptions = catalog.getProcessingOptions();
						procOptions.setAllProcessingOptions(!disableProcessingOptions);
					}
				}

				// BATCHING RELATED CODE START  : The below lines are added to allow the item save as batch. please comment it out if you do not want to use batching 
				catalog.startBatchProcessing();

				int batchedItemCount = 0;
				hmCatalogToBatchedItemCount.put(catalog, batchedItemCount);
				hmCatalogToBatchedItems.put(catalog, new HashMap<String, List<RecordInfo>>());
				//Adding the catalog to the list of catalogs on which batch process has been started
				batchedCatalogs.add(catalog);

				//BATCHING RELATED CODE END
			}	
		}
		return catalog;
	}

	private String getPrimaryKeyPathFromCache(Catalog catalog, String sCatalog)
	{
		//check the hashmap of the catalog and the primary key to get the primarykeypath
		String primaryKeyPath = (String)hmCatalogItemPrimaryKey.get(sCatalog);
		//if primary key path not found
		if (primaryKeyPath == null)
		{
			//get the primary Spec
			primarySpec = catalog.getSpec();
			AttributeDefinition specNode = primarySpec.getPrimaryKeyAttributeDefinition();
			primaryKeyPath = specNode.getPath();
			hmCatalogItemPrimaryKey.put(sCatalog, primaryKeyPath);
		}
		return primaryKeyPath;
	}

	private Spec getSpecFromCache(String specName)
	{
		Spec spec = null;
		if ((spec = hmSpecObj.get(specName)) == null)
		{
			spec = spcManager.getSpec(specName);
			hmSpecObj.put(specName, spec);
		}
		return spec;
	}	

	private AttributeDefinition.Type getTypeFromSpecPathFromCache(String path) 
	{
		AttributeDefinition.Type type = null;
		AttributeDefinition spcNode;
		type = specPathToType.get(path);
		if (type == null)
		{
			Spec spec = null;
			String specName = null;
			if (path.startsWith("/"))
			{
				specName = path.substring(1, path.indexOf("/", 1));
				path = path.substring(1);
			}
			else
			{
				specName = path.substring(0, path.indexOf("/"));
			}
			spec = getSpecFromCache(specName);
			spcNode = spec.getAttributeDefinition(path);
			type = spcNode.getType();
			specPathToType.put(path, type);				
		}
		return type;
	}

	/***
	 * 
	 * Processes the rows with the same record id (ie) the data corresponding a single item
	 * @param jobId � jobId corresponding to the data
	 * @param recordId � recordId corresponding to the data
	 * @param sCatalog � Name of the catalog, where the data needs to be loaded
	 * @param sCategory � Name of the category , where the item needs to be moved
	 * @param hmPathValue � HashMap of the attribute path and its value
	 * @throws PIMAuthorizationException when user is not allowed to perform the operation
	 * @throws SQLException - while loading data from the table
	 * @throws Exception - problem getting the context
	 */    
	public void processRecord(int jobId , int recordId ,String sCatalog, ArrayList<String> sCategory, String actionMode , LinkedHashMap hmPathValue)throws SQLException,Exception
	{ 		   	     
		Catalog catalog = null;    	     
		AttributeDefinition.Type type = null;    	     
		
			
		long timestamp = System.currentTimeMillis();
		RecordInfo recordInfo = new RecordInfo();
		recordInfo.jobId = jobId;
		recordInfo.recordId = recordId;

		if(sCatalog == null)
		{
			sCatalog = defaultCatalog;
		}      

		//get the catalog object			 
		catalog = getCatalogFromCache(sCatalog,true);

		if(catalog == null)
		{				
			logger.logError("Invalid Catalog Name"+sCatalog);
			ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_INVALID_CATALOG,timestamp);
			return ;
		}
		//check the hashmap of the catalog and the primary key to get the primarykeypath 	
		String primaryKeyPath = getPrimaryKeyPathFromCache(catalog, sCatalog);       
		String primaryKeyVal = (String)hmPathValue.get(primaryKeyPath);            
		Item item = null;
		if(primaryKeyVal != null)
		{
			item = catalog.getItemByPrimaryKey(primaryKeyVal);
		}
		if(actionMode.equals(DELETE_ACTION_MODE))
		{        	  
			if(item==null)
			{
				logger.logError(DELETE_FAILED_NO_ITEM);
				ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_ITEM_DOES_NOT_EXIST,timestamp);
				return;
			}
			else if(item.isCheckedOut())
			{
				logger.logError(DELETE_FAILED_ITEM_CHECKED_OUT);				
				ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_ITEM_HAS_CHECKED_OUT_ATTRIBUTES,timestamp);
				return;
			}
			else
			{
				try
				{		
					item.delete();
					flushBatchAndUpdateStatus(catalog, item, recordInfo);
				}
				catch(PIMInternalException e)
				{
					ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_VALIDATION_ERROR,timestamp);
				}
				return;
			}
		}           
		if(actionMode.equals(CREATE_ACTION_MODE)&& item!=null)
		{
			logger.logError(CREATE_FAILED_ITEM_EXISTS);			
			ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_ITEM_ALREADY_EXISTS,timestamp);
			return;
		}            
		if(actionMode.equals(UPDATE_ACTION_MODE)&& item!=null&&item.isCheckedOut())
		{
			logger.logError(UPDATE_FAILED_ITEM_CHECKED_OUT);
			ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_ITEM_HAS_CHECKED_OUT_ATTRIBUTES,timestamp);
			return;
		}        


		if(primaryKeyVal == null)
		{
			if(getTypeFromSpecPathFromCache(primaryKeyPath)!= AttributeDefinition.Type.SEQUENCE )
			{
				logger.logError(STATUS_NO_PRIMARY_KEY);
				ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_NO_PRIMARY_KEY,timestamp);
				return;
			}
			else
			{
				//add to the ArrayList of items with no pk
				listRecordInfoForItemsWithNoPk.add(recordInfo);
			}
		}


		if (item == null)
		{
			item = catalog.createItem();
		}                    
		Set pathKeys = hmPathValue.keySet();
		Iterator it = pathKeys.iterator();                             
		//Handle for different type attributes and set the attribute value within the While loop
		while (it.hasNext()) 
		{
			String path =(String)it.next();
			String strValue =(String)hmPathValue.get(path);                  
			boolean mulOccurence = false;
			String multiValueAttribPath = null;

			//if the path contains the character '#' , then the path is corresponding to a multi occurence attribute
			if((path.indexOf('#')!=-1))
			{
				mulOccurence = true;
				multiValueAttribPath = path;
				path = path.replaceAll("#[0-9]*","");
			}			
			type = getTypeFromSpecPathFromCache(path);
			if(type == null)
			{
				logger.logError("Invalid path information" + path);
				return;
			}
			Object value = null;                                                         
			if(type==AttributeDefinition.Type.RELATIONSHIP)
			{
				try
				{
					AttributeInstance attrInstance = item.getAttributeInstance(path);
					String relValues[] = strValue.split("\\|");
					Catalog relCtg = getCatalogFromCache(relValues[0],false);
					String itemPrimaryKey = relValues[1];             	                 	                  	  
					attrInstance.setValue(relCtg.getItemByPrimaryKey(itemPrimaryKey));
					continue;
				}
				catch(UnsupportedOperationException e)
				{
					exceptionLogger.logError(e);
					ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_SETTING_RELATION_ATTRIBUTE,timestamp);
					return; 
				}
				catch(PIMInvalidPathException e)
				{
					exceptionLogger.logError(e);
					ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_INVALID_PATH,timestamp);
					return; 
				}				
				catch(Exception e)
				{					
					exceptionLogger.logError(e);			
				}

			}                                  
			else if(type==AttributeDefinition.Type.NUMBER||type==AttributeDefinition.Type.NUMBER_ENUMERATION||type==AttributeDefinition.Type.CURRENCY)
			{
				try
				{
					value= new Double(strValue);  
				}
				catch(NumberFormatException e)
				{
					logger.logError("Item with Primary Key"+primaryKeyVal+"has invalid input format specified corresponding to the "+path);
					exceptionLogger.logError(e);
					ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_INVALID_INPUT_FORMAT,timestamp);
					return;
				}   
			}               	  
			else if(type==AttributeDefinition.Type.INTEGER||type==AttributeDefinition.Type.SEQUENCE)
			{
				try
				{
					value= new Integer(strValue);  
				}
				catch(NumberFormatException e)
				{
					logger.logError("Item with Primary Key"+primaryKeyVal+"has invalid input format specifed corresponding to the "+path);
					exceptionLogger.logError(e);
					ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_INVALID_INPUT_FORMAT,timestamp);				    
					return;
				}       
			}   
			else if(type==AttributeDefinition.Type.DATE)
			{
				try
				{
					DateFormat df = new SimpleDateFormat();
					value = df.parse(strValue);
				}
				catch(ParseException e)
				{
					logger.logError("Item with Primary Key"+primaryKeyVal+"has invalid input format specified corresponding to the path"+path);
					exceptionLogger.logError(e);
					ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_INVALID_INPUT_FORMAT,timestamp);
					return;
				}       
			}                  
			else if(type ==AttributeDefinition.Type.BINARY || type ==AttributeDefinition.Type.FLAG || type ==AttributeDefinition.Type.GROUPING|| type ==AttributeDefinition.Type.IMAGE || type ==AttributeDefinition.Type.PASSWORD || type ==AttributeDefinition.Type.IMAGE_URL || type ==AttributeDefinition.Type.STRING || type ==AttributeDefinition.Type.STRING_ENUMERATION || type ==AttributeDefinition.Type.LOOKUP_TABLE || type ==AttributeDefinition.Type.THUMBNAIL_IMAGE || type ==AttributeDefinition.Type.THUMBNAIL_IMAGE_URL || type ==AttributeDefinition.Type.URL || type ==AttributeDefinition.Type.TIMEZONE)
			{
				value= strValue;
				// Added to handle flag datatype as Boolean value
				if(type ==AttributeDefinition.Type.FLAG) {
					Boolean bFlag = new Boolean(strValue);
					value = bFlag;
				}
			}                                      
			try
			{                   
				if(mulOccurence)
				{
					item.setAttributeValue(multiValueAttribPath, value);
					continue;
				}
				item.setAttributeValue(path, value);
			}
			catch(PIMInvalidPathException e)
			{
				logger.logError("Item with Primary Key"+primaryKeyVal+"has invalid path"+path);
				exceptionLogger.logError(e);
				ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,STATUS_INVALID_PATH,timestamp);
				return;  
			}
			catch(UnsupportedOperationException e)
			{   				
				logger.logError("Item with Primary Key"+primaryKeyVal+"tries to do an unsupported operation, use 'setRelatedItem' for relationship attributes");
				exceptionLogger.logError(e);
				ProcessorUtils.updateItemStatus(insertStatusQuery,jobId,recordId,status,timestamp);
				return;  
			}     
		}     		
		ArrayList<Category> categoryList = new ArrayList<Category>();
		if(sCategory.size()>0)
		{
			categoryList = populateCategories(sCategory);                 
		}                     
		item.moveToCategories(categoryList);
		try
		{
			item.save();
		}
		catch(PIMInternalException e)
		{
			logger.logError("Item with Primary Key"+primaryKeyVal+"failed import due to validation errors");
			exceptionLogger.logError(e);
		}

		flushBatchAndUpdateStatus(catalog, item, recordInfo);
	}

	private void flushBatchAndUpdateStatus(Catalog catalog, Item item, RecordInfo recordInfo ) throws SQLException, Exception {
		// The below details are maintained in order to update the status table when items are saved in a batch
		//In case, you do not want to save items as a batch, comment out the corresponding Batching Api calls and get the validation error for each item after item.save() is called
		int itemCountForCatalog = hmCatalogToBatchedItemCount.get(catalog);
		HashMap<String, List<RecordInfo>> hmItemPkToRecordInfo = hmCatalogToBatchedItems.get(catalog);

		List<RecordInfo> list_recordInfo = hmItemPkToRecordInfo.get(item.getPrimaryKey());
		if (list_recordInfo == null)
		{
			list_recordInfo = new ArrayList<RecordInfo>();
		}
		list_recordInfo.add(recordInfo);
		hmItemPkToRecordInfo.put(item.getPrimaryKey(),list_recordInfo);
		hmCatalogToBatchedItemCount.put(catalog,++itemCountForCatalog);

		if (itemCountForCatalog >= itemSaveBatchSize)
		{
			List<ExtendedValidationErrors> validationErrors = null ;
			validationErrors = catalog.flushBatch();
			// reinitialising the counter of the items for each catalog 
			itemCountForCatalog = 0;
			hmCatalogToBatchedItemCount.put(catalog,itemCountForCatalog);
			if(validationErrors != null)
				updateStatusTableForItemSavedAsBatch(validationErrors,catalog);			
		}
	}      

	/***
	 * This method checks out the loaded item into the Collaboration Area mentioned in the property file.
	 * @param item - Item to be checked out
	 * @param colAreaName - The CollaborationAreaName can be hardcoded or mentioned like this extn://fully qualified name of the implementation class of the WorkflowExtension.
	 * @throws PIMAuthorizationException - when user is not Aythorized to do the operation
	 * @throws Exception - problem getting the context
	 */
	private void checkoutImportedItems(Item item, String colAreaName)throws Exception
	{ 	 		
		String EXTN = "extn:";
		if(colAreaName.startsWith(EXTN))
		{
			String workflowIdentifierImpl = colAreaName.substring(5);
			colAreaName = ProcessorUtils.getCollaborationAreaforItem(item,workflowIdentifierImpl);
			collabExtn = true;
		} 	 		

		CollaborationArea colArea;
		if((colArea=hmCollaborationAreaObj.get(colAreaName))==null)
		{       	 			  		 
			colArea = caMgr.getCollaborationArea(colAreaName);
			if(colArea!= null)
			{
				hmCollaborationAreaObj.put(colAreaName, colArea);        
				//maintain a hashmap of collaboration Area objects 		 
				itmColArea = (ItemCollaborationArea)colArea;
			}
			else
			{
				logger.logError("Invalid Collaboration Area name"+colAreaName); 
				return;
			}   
		}

		if(collabExtn)
		{
			itmColArea.checkout(item);
		}
		else
		{ 		 
			if(checkoutItems.size()>=checkoutItemSize)
			{
				batchCheckout();
			}
			checkoutItems.add(item);
		} 		 
	}	
	/***
	 * This method is used to checkout a collection of items
	 * @throws PIMAuthorizationException
	 */
	protected void batchCheckout() 
	{
		if(itmColArea!= null)
		{
			itmColArea.checkout(checkoutItems);
			checkoutItems.clear();
		}
	}

	/***
	 * This method is used to pupulate an Hashmap of Category path and Category Object
	 * @param categories - ArrayList of category names
	 * @return - ArrayList of Catgeory Objects based on the Category names.
	 * @throws PIMAuthorizationException - when user is not Aythorized to do the operation
	 */
	private ArrayList<Category> populateCategories(ArrayList<String> categories)
	{ 
		ArrayList<Category> list= new ArrayList<Category>();
		Category category=null;
		for(int i=0;i<categories.size();i++)
		{
			String categoryPath = categories.get(i);
			//check if category object is already available based on the path 
			category = hmPathToCategories.get(categoryPath);		     
			if(category == null)
			{
				String names[]= categoryPath.split("\\|"); 			
				Hierarchy hier = null;				
				hier = hierMgr.getHierarchy(names[0]);

				if(hier == null)
				{
					logger.logError("Hierarchy is nonexistent"+names[0]);
				}
				category = hier.getCategoryByPath(names[1]);
				if(category == null)
				{
					logger.logError("Category  is nonexistent"+names[1]);
				} 
				//Populate the Category Path and the Category Object in the HashMap 
				hmPathToCategories.put(categoryPath, category);
			} 
			if(category != null)
			{	
				list.add(category);
			}    
		}    	  		
		return list;
	}

	//The method is added to update the status table when the items are saved in a batch
	//mapping between the returned validation errors and the RecordInfo fo r the item is mapped using the item primary key.
	//When an Item has no primary key , then it is not returned as a validation error during batching, 
	//So look into the logs if the item fails import due to missing primary key
	private void updateStatusTableForItemSavedAsBatch(List<ExtendedValidationErrors> validationErr,Catalog catalog)throws SQLException,Exception
	{					
		long timestamp = System.currentTimeMillis();
		HashMap<String, List<RecordInfo>> hmItemPkToRecordInfo = hmCatalogToBatchedItems.get(catalog);
		List<ValidationError> itemValidationError = null;
		for (ExtendedValidationErrors err :validationErr)
		{	        
			Item batchedItem =(Item)err.getSource(); 
			itemValidationError = err.getErrors();

			if (itemValidationError.isEmpty())
			{
				status = STATUS_ITEM_SAVED;
				logger.logError("Item with Primary Key"+batchedItem.getPrimaryKey()+"was saved successfully");
				if((collaborationAreaName!= null)&&!collaborationAreaName.equals("null")&&!collaborationAreaName.equals(""))
				{
					checkoutImportedItems(batchedItem,collaborationAreaName);
				}
			}
			else
			{
				status = STATUS_VALIDATION_ERROR;
				logger.logError("Item with Primary Key"+batchedItem.getPrimaryKey()+"failed import due to validation errors ");
			}      			
			RecordInfo itemRecordInfo = null;
			List<RecordInfo> current_list_recordInfo = hmItemPkToRecordInfo.get(batchedItem.getPrimaryKey());
			for(int i=0; i<current_list_recordInfo.size();i++)
			{
				itemRecordInfo = current_list_recordInfo.get(i);
				ProcessorUtils.updateItemStatus(insertStatusQuery,itemRecordInfo.jobId,itemRecordInfo.recordId,status,timestamp);
			}		

		}
		for(RecordInfo recInfo:listRecordInfoForItemsWithNoPk)
		{
			status = STATUS_CHECK_STATUS_IN_LOGS;
			ProcessorUtils.updateItemStatus(insertStatusQuery,recInfo.jobId, recInfo.recordId,status,timestamp);
		}
		hmItemPkToRecordInfo.clear();
	}


}
