/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/



/**
 *  Copyright IBM Corp. and other(s) 2000, 2008 All Rights Reserved. 
 *
 * The following sample of source code ("Sample Code") is owned by International Business Machines Corporation or 
 * one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.
 * You may use, copy, modify, and distribute the Sample Code in any form without payment to IBM, for the purpose of assisting you in the development of your applications.

 * The Sample Code is provided to you on an "AS IS" basis, without warranty of any kind. 
 * IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * Some jurisdictions do not allow for the exclusion or limitation of implied warranties, so the above limitations or exclusions may not apply to you. 
 * IBM shall not be liable for any damages you suffer as a result of using, copying, modifying or distributing the Sample Code, even if IBM has been advised of the possibility of such damages.
 **/
package com.ibm.ccd.loader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.KeyStore.LoadStoreParameter;
import java.sql.SQLException;

import com.ibm.pim.context.Context;
import com.ibm.pim.context.PIMContextFactory;
import com.ibm.pim.extensionpoints.ReportGenerateFunction;
import com.ibm.pim.extensionpoints.ReportGenerateFunctionArguments;
import com.ibm.pim.utils.Logger;

/***
 * 
 * This class facilitates the invocation of the Data Loader program using reports
 *
 */
public class LoaderReportGenerateFunction implements ReportGenerateFunction{
	ReportGenerateFunctionArguments inArgs = null;


	private static final String LOGGER_NAME ="JavaAPI_Exception";

	DataLoader dataLoader;
	Context context;  
	Logger logger;

	/**
	 * Generate a Report. 
	 * @param inArgs the arguments for this invocation.
	 */
	public void reportGenerate(ReportGenerateFunctionArguments inArgs)
	{

		try
		{
			context =PIMContextFactory.getCurrentContext();
			logger = context.getLogger(LOGGER_NAME);
			this.inArgs = inArgs;
			Integer jobId = null;       
			Integer startRecordId = null;
			Integer stopRecordId = null;           
			String propertyFile = null;     
			String dataFile = null;       
			String option = null;
			Object value = null;  		
			if((value=inArgs.getInputs().get(LoaderJobInvoker.PROPERTY_FILENAME))!= null && !value.equals(""))
			{
				propertyFile = (String)(value);
			}			
			if((value=inArgs.getInputs().get(LoaderJobInvoker.DATA_FILENAME))!= null && !value.equals(""))
			{
				dataFile = (String)(value);
			}
			if((value=inArgs.getInputs().get(LoaderJobInvoker.OPTION))!= null && !value.equals(""))
			{
				option = (String)(value);
			}		
			if((value=inArgs.getInputs().get(LoaderJobInvoker.JOB_ID))!= null && !value.equals(""))
			{
				jobId = new Integer((String)(value));       
			}
			if((value=inArgs.getInputs().get(LoaderJobInvoker.START_RECORD_ID))!= null && !value.equals(""))
			{
				startRecordId = new Integer((String)(value));
			}
			if((value=inArgs.getInputs().get(LoaderJobInvoker.STOP_RECORD_ID))!= null && !value.equals(""))
			{
				stopRecordId =new Integer((String)(value));
			}
			dataLoader = new DataLoader();
			dataLoader.createLoader( propertyFile , dataFile , option, jobId , startRecordId , stopRecordId );
		}
		catch(FileNotFoundException e)
		{
			logger.logError(e);
			return;
		}
		catch(IOException e)
		{
			logger.logError(e);
			return;
		}
		catch(SQLException e)
		{
			logger.logError(e);
			return;
		}
		catch (NumberFormatException e)
		{
			logger.logError(e);
			return;
		}
		catch(Exception e)
		{
			logger.logError(e);
			return;
		}	
	}
}


