/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/




/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;


/*
 * Represents the Boolean values
 * limitation : The value is represented as String and doesn't support formats
 */
public class FlagAttribute extends StringEnumAttribute {
    private static final String[] flagEnum = {"", "false", "true" };

    public FlagAttribute(String name, boolean editable, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, flagEnum, editable, minimumOccurrence, maximumOccurrence,
            displayName);
        removeValues();
        setValue(flagEnum[0]);
    }

    public FlagAttribute(String name, String defaultValue, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, flagEnum, defaultValue, editable, minimumOccurrence,
            maximumOccurrence, displayName);
        removeValues();

        if (defaultValue != null) {
            setValue(defaultValue);
        } else {
            setValue(flagEnum[0]);
        }
    }

    public FlagAttribute(String name, String defaultValue, boolean editable,
        int minimumLength, int maximumLength, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, flagEnum, defaultValue, editable, minimumLength,
            maximumLength, minimumOccurrence, maximumOccurrence, displayName);
        removeValues();

        if (defaultValue != null) {
            setValue(defaultValue);
        } else {
            setValue(flagEnum[0]);
        }
    }

    public int getType() {
        return AttributeTypes.FLAG;
    }

    protected void checkValue(String value) {
        if ((enumValues == null) || (enumValues.length == 0)) {
            throw new IllegalArgumentException("Enum is Empty/Not defined for " +
                getName());
        }

        for (int i = 0; i < enumValues.length; ++i) {
            if (enumValues[i].equalsIgnoreCase(value)) {
                return;
            }
        }

        throw new IllegalArgumentException(" Not defined Enum for " + value +
            "in " + getName());
    }
    
	public String[] getEnumValues() {
		return flagEnum;
	}

}
