/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/




/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.taglib.dataobjects;

import java.util.HashMap;

import com.ibm.ccd.ui.dataobjects.NVPair;


/**
* SearchElement which represents the Select Search Objects
* @author  IBM India
* @version 1.0
* @since   Iteration 2
**/
public class SelectSearchElement extends BaseSearchElement implements Cloneable {
    /**
     * Identifier for default index
     */
    private int defValueIndex;

    /**
     * The possible selectabale values , each of type TagAttributeData
     */
    private NVPair [] possibleValues;

    /**
     * Identifier for selected value index
     */
    private int selectedValueIndex;

    /**
     *Construcs an emptry SelectSearchElement object
     **/
    public SelectSearchElement() {
        super(ISearchElementTypesEnum.SELECT);
    }

    /**
     * Constructs the SelectSearchElement for the specified name, defaault value index,possible values, default operator and  operatormap
     *  This constructor sets the mandatory presence as true and defaultable as false.
     * @param aElementName - The name of the select search element
     * @param aDefaultValueIndex - The default selected Index
     * @param possibleValues - The possible select Values, whcih should contain the tagAttribueData Objects
     * @param aDefaultOperator - The default operator
     * @param aOperatorsMap - The available operators map
     **/
    public SelectSearchElement(String aElementName, int aDefaultValueIndex,
        NVPair[] possibleValues, String aDefaultOperator, HashMap aOperatorsMap) {
        this(aElementName, aDefaultValueIndex, possibleValues,
            aDefaultOperator, aOperatorsMap, true, false);
    }

    /**
     * Constructs the SelectSearchElement for the specified name, defaault value index,possible values, default operator,operatormap ,mandatory and defaultable flags
     * @param aElementName - The name of the select search element
     * @param aDefaultValueIndex - The default selected Index
     * @param possibleValues - The possible select Values, whcih should contain the tagAttribueData Objects
     * @param aDefaultOperator - The default operator
     * @param aOperatorsMap - The available operators map
     * @param aIsMandatory - The mandatory presence of this element
     * @param aIsDefaultable - The defaultability of this element
     **/
    public SelectSearchElement(String aElementName, int aDefaultValueIndex,
        NVPair [] possibleValues, String aDefaultOperator, HashMap aOperatorsMap,
        boolean aIsMandatory, boolean aIsDefaultable) {
        super(ISearchElementTypesEnum.SELECT, aElementName, aDefaultOperator,
            aOperatorsMap, aIsMandatory, aIsDefaultable);

        this.possibleValues = possibleValues;
        this.defValueIndex = aDefaultValueIndex;
    }

    /**
     * getting the value
     * @return Object
     */

    /*
            public Object getValue() {
                    return (possibleValuesList != null)
                            ? possibleValuesList.get(selectedValueIndex)
                            : null;
            }
    */

    /**
     * assigning the index
     * @param aSelectedValueIndex index value
     */
    public void setSelectedIndex(int aSelectedValueIndex) {
        this.selectedValueIndex = aSelectedValueIndex;
    }

    /**
     * assigning the index value
     * @param aSelectedValueIndexStr index value
     */
    public void setValue(String aSelectedValueIndexStr) {
        try {
            this.selectedValueIndex = Integer.parseInt(aSelectedValueIndexStr);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * getting the default value
     * @return Object
     */

    /*        public Object getDefaultValue() {

                    return (possibleValuesList != null)
                            ? possibleValuesList.get(defValueIndex)
                            : null;
            }*/

    /**
     * assign the default index
     * @param aDefaultValueIndex
     */
    public void setDefaultIndex(int aDefaultValueIndex) {
        this.defValueIndex = aDefaultValueIndex;
    }

    /**
     * Gets the possibleValuesList
     * @return Returns a ArrayList
     */
    public NVPair [] getPossibleValues() {
        return possibleValues;
    }

    /**
     * Sets the possibleValuesList
     * @param possibleValues The possibleValuesList to set
     */
    public void setPossibleValues(NVPair [] possibleValues) {
        this.possibleValues = possibleValues;
    }

    /**
     * get the seletec index
     * @return  int
     */
    public int getSelectedIndex() {
        return selectedValueIndex;
    }

    /**
     * get the default index
     * @return int
     */
    public int getDefaultIndex() {
        return defValueIndex;
    }

    /* (non-Javadoc)
     * @see com.ibm.ccd.ui.taglib.dataobjects.BaseSearchElement#clone()
     */
    public Object clone() throws CloneNotSupportedException {
        SelectSearchElement element = null;
        element = (SelectSearchElement) super.clone();
        element.possibleValues = (NVPair[]) possibleValues.clone();
        return element;
    }
}
