/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/




/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.tags.element;

import org.apache.jetspeed.portlet.PortletConfig;
import org.apache.jetspeed.portlet.PortletContext;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.util.UIUtils;


public class NumberEnumerationElement extends BaseElement {
    private PortletRequest portletRequest = null;
    private PortletContext portletContext = null;
    private NumberEnumAttribute numberEnumAttribute = null;

    public NumberEnumerationElement(Attribute attribute, PortletURI portletURI) {
        super(attribute, portletURI);
        numberEnumAttribute = (NumberEnumAttribute) attribute;
    }

    protected String replaceSpecialCharacters(String str){
		if(str == null)
			return str;
		if(str.equals("&nbsp;"))
			return "&amp;nbsp;";
		else if(str.equals("&gt;"))
			return "&amp;gt;";
		else if(str.equals("&lt;"))
			return "&amp;lt;";
			return str;
	}
    protected String getLocalizedNumber(String value) {
		//Hack for "null" value
		if(value != null){
			if(value.equals("null"))
				return "";
			
		}
		if(UIUtils.isEmpty(value)) return "";
		
		PortletRequest portletRequest = (PortletRequest) pageContext.getAttribute("portletRequest");
		return UIUtils.formatNumber(value,portletRequest.getLocale());

	}
  
    
    public String renderElement() {
        portletRequest = (PortletRequest) pageContext.getAttribute(
                "portletRequest");
        portletContext = ((PortletConfig) pageContext.getAttribute(
                "portletConfig")).getContext();

        StringBuffer buffer = new StringBuffer();

        if (attribute.isMultiValued()) {
            for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
            	String path = getAttributeID(String.valueOf(i));
            	String error = attribute.getValidationError(path);
				if(error != null){
					if(isShowLabel())
						buffer.append(displayError(error));
				}
				else{
					if(isShowLabel())
						buffer.append(displayLabel());
				}
                buffer.append(displayField(i, getStringValue(attribute, i)));
            }
        } else {
        	String path = getAttributeID(String.valueOf(0));
        	String error = attribute.getValidationError(path);
			if(error != null){
				if(isShowLabel())
					buffer.append(displayError(error));
			}
			else{
				if(isShowLabel())
		            buffer.append(displayLabel());
			}
            buffer.append(displayField(0, getStringValue(attribute, 0)));
        }

        return buffer.toString();
    }

    /**
     * @param id
     * @param value
     * @return string
     */
    public String displayField(long id, Object value) {
        StringBuffer buffer = new StringBuffer();
        String newValue = replaceSpecialCharacters((String)value);  // Added new
		value = newValue;	
        String formattedNumber = null;
		String path = getAttributeID(String.valueOf(id));
		if(attribute.getValidationError(path) == null){
			formattedNumber = getLocalizedNumber(value+"");
			
			
		}else {
			
        	formattedNumber = (String)value;
		}
		addHTMLProperty(HTMLVALUE,formattedNumber) ;
		attribute.setPath(path);
		String path1 = path.substring(0,path.lastIndexOf('_'));
		addHTMLProperty(JS_EVENT_ONCHANGE,"validateText('"+path1+"')");   
        addHTMLProperty(HTMLID,path);
        addHTMLProperty(HTMLNAME,path);
        buffer.append(HTML_ATTRIBUTE_START);
        if (isEditable()) {
            double[] doubleArr = numberEnumAttribute.getEnumDb();
            buffer.append(HTML_SPACE).append(HTML_SELECT)
                  .append(getHTMLValue(HTMLSTYLECLASS))
                  .append(getHTMLValue(HTMLID)).append(getHTMLValue(HTMLNAME));
				if (isShowLabel())
					buffer.append(getHTMLValue(JS_EVENT_ONCHANGE));
                  buffer.append(HTML_CLOSE_TAG)
                  .append(generateHTMLOption(doubleArr,
                    (String)formattedNumber)).append(HTML_SELECT_CLOSE);//changed value to formattedNumber

            if (attribute.isMultiValued()) {
            	if(isShowLabel())
            		buffer.append(displayPlusMinus(id));
            }
        } else {
			if(attribute.isMultiValued()) {
				buffer.append(displayOccurrenceNumber(id));	
			}        	
          buffer.append(formattedNumber) ;
        }

        buffer.append(HTML_ATTRIBUTE_END);

        return buffer.toString();
    }

    /**
     * generates the HTML options for HTML Select tag
     *
     * @param values
     * @param selectedValue
     * @return string
     */
    protected String generateHTMLOption(double[] values, String selectedValue) {
        StringBuffer buffer = new StringBuffer();

        if (values != null) {
            //Displays the HTML option value NONE
            buffer.append(generateHTMLNoneOption());

            //Displays the HTML select option values 		
            for (int i = 0; i < values.length; i++) {
                addHTMLProperty(HTMLVALUE, values[i]+"");

                if (String.valueOf(getLocalizedNumber(values[i]+"")).equals(selectedValue)) {
                    buffer.append(HTML_OPTION_OPEN)
                          .append(getHTMLValue(HTMLVALUE) + " " +
                        HTML_SELECTED + " " + HTML_CLOSE_TAG).append(getLocalizedNumber(values[i]+""))
                          .append(HTML_OPTION_CLOSE);
                } else {
                    buffer.append(HTML_OPTION_OPEN)
                          .append(getHTMLValue(HTMLVALUE) + HTML_CLOSE_TAG)
                          .append(getLocalizedNumber(values[i]+"")).append(HTML_OPTION_CLOSE);
                }
            }
        } else {
            buffer.append(generateHTMLNoneOption());
        }

        return buffer.toString();
    }

    /**
     * generate HTML option with none as default selected
     * @return HTML None Option value eg., <option value="-1">NONE</option>
     */
    protected String generateHTMLNoneOption() {
        StringBuffer buffer = new StringBuffer();

		buffer.append(HTML_OPTION_OPEN).append(HTMLVALUE + HTML_OPEN_QUOTE + "" + HTML_CLOSE_QUOTE + HTML_CLOSE_TAG)
		.append(UIUtils.getTextLabel(portletContext, portletRequest,
                "DROPDOWN_VALUE_NONE")).append(HTML_OPTION_CLOSE);

        return buffer.toString();
    }

    /**
     * @param attribute
     * @param index
     * @return string
     */
    protected String getStringValue(Attribute attribute, int index) {
        NumberEnumAttribute numberEnumAttrib = (NumberEnumAttribute) attribute;

        if (numberEnumAttrib.getCurrentOccurrence() > 0) {
            if (numberEnumAttrib.isMultiValued()) {
                String[] values = numberEnumAttrib.getValues();

                return values[index]+"";
            } else {
                return numberEnumAttrib.getValue()+"";
            }
        } else {
            return numberEnumAttrib.getDefaultValue()+"";
        }
    }

}
