#!/bin/bash
# IBM Confidential
# OCO Source Materials
# 5724-V51
#
# (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.


# scan log file and report any errors found
reportErrors ()
{
# expected oracle errors
    ORA_TABLE_DOES_NOT_EXIST="ORA-00942"
    ORA_INDEX_DOES_NOT_EXIST="ORA-01418"
    ORA_SYNONYM_DOES_NOT_EXIST="ORA-01434"
    ORA_USER_DOES_NOT_EXIST="ORA-01918"
    ORA_SEQUENCE_DOES_NOT_EXIST="ORA-02289"


    ERRORS=$(cat $tmplog | grep -n "ORA-" | grep -v ${ORA_TABLE_DOES_NOT_EXIST} \
	| grep -v ${ORA_INDEX_DOES_NOT_EXIST} \
	| grep -v ${ORA_SYNONYM_DOES_NOT_EXIST} \
	| grep -v ${ORA_USER_DOES_NOT_EXIST} \
	| grep -v ${ORA_SEQUENCE_DOES_NOT_EXIST})


    if [ ! -z "$ERRORS" ]; then
	
	${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
	${ECHO} COMMON_INFO_CREATE_SCHEMA_FAILED_SH
	${ECHO} COMMON_INFO_CREATE_SCHEMA_FAILED_ADVICE_SH
	${ECHO} ""	
	${ECHO} COMMON_INFO_CHECK_LOG_FILE_SH
	${ECHO} "  $tmplog"            
	${ECHO} ""            
	${ECHO} !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	
	${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"  >> $tmplog
	${ECHO} "Create schema failed." >> $tmplog
	${ECHO} "Please make sure that no user/application is connected to the database." >> $tmplog
        ${ECHO} "" >> $tmplog	
        ${ECHO} "Found following errors:" >> $tmplog
	${ECHO} "" >> $tmplog
	${ECHO} "$ERRORS" >> $tmplog
	${ECHO} "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" >> $tmplog

	exit 1;
    fi
}

cd $TOP
${MKDIR} logs >/dev/null
cd logs
${TOUCH} schema.log >/dev/null

tmplog=${TOP}/logs/schema.log

# clear log file
>"$tmplog"

${ECHO} "========================================================"  >> $tmplog
${ECHO} "Event: Creating schema starts  at `date` by  $LOGNAME" >> $tmplog


${ECHO}
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO} "**"
${ECHO} COMMON_INFO_CREATE_SCHEMA_SH
${ECHO} COMMON_INFO_WIPED_OUT_SH
${ECHO} "**"
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO}

. ${TOP}/src/db/schema/util/set_db_localized.sh "$@"

${ECHO}
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO} "**"
${ECHO} COMMON_INFO_CONFIRM_SH
${ECHO} "**"
${ECHO} "********************************************************"
${ECHO} "********************************************************"
${ECHO}

. ${TOP}/src/db/schema/util/set_db_localized.sh "$@"


${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_DROPPING_SCHEMA_SH
${ECHO} ====================================================
${ECHO}

#${TOP}/src/db/schema/util/wipe_schema.sh --confirm_db=no >> $tmplog

reportErrors

. ${TOP}/src/db/schema/util/drop_temp_agg_tables.sh --confirm_db=no >> $tmplog
. ${TOP}/src/db/schema/util/drop_log_tables_seq.sh --confirm_db=no >> $tmplog
${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_CREATE_TABLE_SH
${ECHO} ====================================================
${ECHO}

cd $DB_ROOT_DIR/schema

#                     declared in pa.rc

for DDL in `ls gen/ddl_*.sql 2>/dev/null`
do
  ${ECHO}
  ${ECHO} ====================================================
  ${ECHO} COMMON_INFO_RUN_SQL_SH
  ${ECHO} $DDL
  ${ECHO} ====================================================
  ${ECHO}
  . ${DB_DIR}/util/run_db_cmd.sh --file=$DDL --errorfilter=yes >> $tmplog

done


if [ "$AUSTIN_MODE" = "dev" -o "$AUSTIN_MODE" = "qa" ]; then
${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_BUILD_CONSTRAINTS_SH
${ECHO} ====================================================
${ECHO}

. ${DB_DIR}/util/run_db_cmd.sh --file=${DB_DIR}/sanitychecks/create_devonly_constraints.sql --errorfilter=yes >> $tmplog


fi


${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_CREATE_PLAN_TABLE_SH
${ECHO} ====================================================
${ECHO}
. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/ddl_performance.sql --errorfilter=yes >> $tmplog



${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_CREATE_DB_OBJ_SH
${ECHO} ====================================================
${ECHO}

. ${TOP}/src/db/schema/util/create_db_functions_localized.sh

${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_LOAD_SP_SH
${ECHO} ====================================================
${ECHO}

for FILE in `ls $TOP/src/db/schema/gen/proc*.sql 2>/dev/null`
do

. ${DB_DIR}/util/run_db_cmd.sh --file=$FILE --disconnect=yes --errorfilter=yes >> $tmplog


done

${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_POPULATE_DATA_SH
${ECHO} ====================================================
${ECHO}

. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/dml_pgc_page_customization.sql --errorfilter=yes >> $tmplog

. ${DB_DIR}/util/run_db_cmd.sh --file=$DB_SCRIPTS/dml_per_personalization.sql --errorfilter=yes >> $tmplog


${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_BUILD_IDX_SH
${ECHO} ====================================================
${ECHO}
#
# Re-create the indexes.
# We want to create the indexes after populating the database to make
# the inserts faster.
# Note: Dropping a table automatically drops any indexes associated with the table.
#
for FILE in `ls $TOP/src/db/schema/gen/idx*.sql 2>/dev/null`
do
${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_RUN_SQL_IDX_SH
${ECHO} " $FILE"
${ECHO} ====================================================
${ECHO}

. ${DB_DIR}/util/run_db_cmd.sh --file=$FILE --errorfilter=yes >> $tmplog


done


#cd $TOP
#${MKDIR} logs
#cd logs
#${TOUCH} system_default.log

cd ${TOP}/src/db/schema/create

# ${ECHO} ====================================================
# ${ECHO} COMMON_INFO_SP_SH
# ${ECHO} ====================================================

# for FILE in `ls $TOP/src/db/schema/proc/*.sql 2>/dev/null`
# do
# echo "file is $FILE"
# ${CCD_SQLPLUS} ${CONNECTION} <<EOF
# start $FILE
# EOF
# done

${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_PRIVILEGES_SH
${ECHO} ====================================================
${ECHO}
${JAVA_RT} com.ibm.ccd.common.generate.GenObjectTypeInfoToDb >> $tmplog

${ECHO}
${ECHO} ====================================================
${ECHO} UPDATE_DATABASE_VERSION
${ECHO} ====================================================
${ECHO}
${JAVA_RT} com.ibm.ccd.synchronize.DBSchemaVersion --autoupd >> $tmplog

${ECHO}
${ECHO} ====================================================
${ECHO} COMMON_INFO_CREATE_MKT_SH
${ECHO} ====================================================
${ECHO}

${TOP}/src/db/schema/trigo/create_ccd.sh --confirm_db=no >> $tmplog

${ECHO} ====================================================
${ECHO} COMMON_INFO_ANALYZING_SH
${ECHO} ====================================================

#
# SHOULD THIS CONDITION BE CHANGED TO $CCD_DB?
#
if [ "$TRIGO_DB" = "oracle" ]; then

. ${DB_DIR}/util/run_db_cmd.sh --file=${DB_DIR}/create/preanalyze.sql --errorfilter=yes >> $tmplog

fi

if [ "$CCD_DB" = "db2" ]; then

. ${DB_DIR}/util/run_db_cmd.sh --file=${DB_DIR}/dbscripts/db2/create_index_explain_object.sql --errorfilter=yes >> $tmplog

fi



cd ${TOP}/src/db/schema/util
./analyze_schema.sh --confirm_db=no >> $tmplog


${ECHO} ====================================================
${ECHO} COMMON_INFO_DONE_SH
${ECHO} ====================================================


${ECHO} "Event: Creating schema ends  at `date` by  $LOGNAME" >> $tmplog
${ECHO} "========================================================" >> $tmplog

reportErrors

exit 0;


